﻿using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware;
using SecretLabs.NETMF.Hardware.Netduino;
using System.IO.Ports;

namespace Parallax125kHzRFIDReader
{
    public class Program
    {
        //This will allow us to turn the reader on and off 
        static OutputPort rfidON = new OutputPort(Pins.ONBOARD_LED, true);

        public static void Main()
        {
            //On Netduino pin 0 (RX Pin) 
            SerialPort SPort = new SerialPort("COM1", 9600, System.IO.Ports.Parity.None, 8, System.IO.Ports.StopBits.One);

            //We don't want to read forever 
            SPort.ReadTimeout = 500;
            SPort.WriteTimeout = 500;

            //Turn on the reader 
            rfidON.Write(false);

            SPort.DataReceived += new SerialDataReceivedEventHandler(SPort_DataReceived);

            //Open the Serial Port for communication (.NetMF 4.1 requirement) 
            SPort.Open();

            //Loop it forever! 
            while (true)
            {

            }
        }

        static void SPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            //This aray will hold our Cards RFID tag (all 12 bytes) 
            byte[] buf = new byte[16];
            byte[] trash = new byte[64];

            //Read the card, store in the array "buf" with no offset and a max read of 12 bytes) 
            int readcnt = ((SerialPort)sender).Read(buf, 0, 16);

            if (readcnt == 16 && buf[0] == 0x02 && buf[13] == 13 && buf[14] == 10 && buf[15] == 0x03)
            {
                //Turn off the reader 
                rfidON.Write(true);

                // Get rid of extra data in the pipeline
                while (((SerialPort)sender).Read(trash, 0, 64) > 0)
                {
                }

                //This will hold the cards ID Tag (10 bytes) 
                string CardId = "";

                // Strip away the check sum
                for (int i =1; i <= 10; i++)
                {
                    if (buf[i] != 0x02 && buf[i] != 0x13 && buf[i] != 0x10 && buf[i] != 0x03)
                    {
                        CardId += (char)buf[i];
                    }
                }

                /*Check if the CardID length is 
                 * equal to 10. Remember, RFID's  
                 * are 12 characters. One "start" character, 
                 * one end character, and 10 characters (ID)*/
                if (CardId.Length == 10)
                {
                    //Show the ID 
                    Debug.Print(CardId);

                    //Clear the ID 
                    CardId = "";

                    //Clear the array 
                    Array.Clear(buf, 0, 16);

                    //Clear the "byte" counter 
                    readcnt = 0;

                    /*Sleep for two seconds, otherwise 
                     * you would get a bunch of  
                     * duplicate reads*/
                    Thread.Sleep(1000);

                    //Enable the reader 
                    rfidON.Write(false);
                }
                else
                {
                    rfidON.Write(false);
                }
            }
        }
    }
}
